@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 #############################################################################
 #  Copyright Kitware Inc.
 #
@@ -21,11 +19,11 @@ import hashlib
 import math
 import os
 import re
-import six
+import urllib
 
 from girder.api import access, filter_logging
 from girder.api.v1.item import Item as ItemResource
-from girder.api.describe import describeRoute, Description
+from girder.api.describe import autoDescribeRoute, describeRoute, Description
 from girder.api.rest import filtermodel, loadmodel, setRawResponse, setResponseHeader
 from girder.exceptions import RestException
 from girder.models.model_base import AccessType
@@ -119,6 +117,8 @@ class TilesItemResource(ItemResource):
                            self.getAssociatedImagesList)
         apiRoot.item.route('GET', (':itemId', 'tiles', 'images', ':image'),
                            self.getAssociatedImage)
+        apiRoot.item.route('GET', (':itemId', 'tiles', 'images', ':image', 'metadata'),
+                           self.getAssociatedImageMetadata)
         apiRoot.item.route('GET', ('test', 'tiles'), self.getTestTilesInfo)
         apiRoot.item.route('GET', ('test', 'tiles', 'zxy', ':z', ':x', ':y'),
                            self.getTestTile)
@@ -143,9 +143,27 @@ class TilesItemResource(ItemResource):
         .param('fileId', 'The ID of the source file containing the image. '
                          'Required if there is more than one file in the item.',
                required=False)
+        .param('force', 'Always use a job to create the large image.',
+               dataType='boolean', default=False, required=False)
         .param('notify', 'If a job is required to create the large image, '
                'a nofication can be sent when it is complete.',
                dataType='boolean', default=True, required=False)
+        .param('tileSize', 'Tile size', dataType='int', default=256,
+               required=False)
+        .param('compression', 'Internal compression format', required=False,
+               enum=['none', 'jpeg', 'deflate', 'lzw', 'zstd', 'packbits', 'webp', 'jp2k'])
+        .param('quality', 'JPEG compression quality where 0 is small and 100 '
+               'is highest quality', dataType='int', default=90,
+               required=False)
+        .param('level', 'Compression level for deflate (zip) or zstd.',
+               dataType='int', required=False)
+        .param('predictor', 'Predictor for deflate (zip) or lzw.',
+               required=False, enum=['none', 'horizontal', 'float', 'yes'])
+        .param('psnr', 'JP2K compression target peak-signal-to-noise-ratio '
+               'where 0 is lossless and otherwise higher numbers are higher '
+               'quality', dataType='int', required=False)
+        .param('cr', 'JP2K target compression ratio where 1 is lossless',
+               dataType='int', required=False)
     )
     @access.user
     @loadmodel(model='item', map={'itemId': 'item'}, level=AccessType.WRITE)
@@ -161,10 +179,14 @@ class TilesItemResource(ItemResource):
         largeImageFile = File().load(largeImageFileId, force=True, exc=True)
         user = self.getCurrentUser()
         token = self.getCurrentToken()
+        notify = self.boolParam('notify', params, default=True)
+        params.pop('notify', None)
         try:
             return self.imageItemModel.createImageItem(
                 item, largeImageFile, user, token,
-                notify=self.boolParam('notify', params, default=True))
+                createJob='always' if self.boolParam('force', params, default=False) else True,
+                notify=notify,
+                **params)
         except TileGeneralException as e:
             raise RestException(e.args[0])
 
@@ -206,6 +228,9 @@ class TilesItemResource(ItemResource):
         for entry in typeList:
             key, dataType, outkey1, outkey2 = (list(entry) + [None] * 2)[:4]
             if key in params:
+                if dataType == 'boolOrInt':
+                    dataType = bool if str(params[key]).lower() in (
+                        'true', 'false', 'on', 'off', 'yes', 'no') else int
                 try:
                     if dataType is bool:
                         results[key] = str(params[key]).lower() in (
@@ -241,7 +266,7 @@ class TilesItemResource(ItemResource):
         If requested, set the content disposition and a suggested file name.
 
         :param item: an item that includes a name.
-        :param contentDisposition: either 'inline' or 'attachemnt', otherwise
+        :param contentDisposition: either 'inline' or 'attachment', otherwise
             no header is added.
         :param mime: the mimetype of the output image.  Used for the filename
             suffix.
@@ -255,10 +280,10 @@ class TilesItemResource(ItemResource):
         if subname:
             filename += '-' + subname
         filename += '.' + MimeTypeExtensions[mime]
-        if not isinstance(filename, six.text_type):
+        if not isinstance(filename, str):
             filename = filename.decode('utf8', 'ignore')
         safeFilename = filename.encode('ascii', 'ignore').replace(b'"', b'')
-        encodedFilename = six.moves.urllib.parse.quote(filename.encode('utf8', 'ignore'))
+        encodedFilename = urllib.parse.quote(filename.encode('utf8', 'ignore'))
         setResponseHeader(
             'Content-Disposition',
             '%s; filename="%s"; filename*=UTF-8\'\'%s' % (
@@ -668,8 +693,15 @@ class TilesItemResource(ItemResource):
                enum=['0', '1', '2'], dataType='int', default='0')
         .param('tiffCompression', 'Compression method when storing a TIFF '
                'image', required=False,
-               enum=['raw', 'tiff_lzw', 'jpeg', 'tiff_adobe_deflate'])
+               enum=['none', 'raw', 'lzw', 'tiff_lzw', 'jpeg', 'deflate',
+                     'tiff_adobe_deflate'])
         .param('style', 'JSON-encoded style string', required=False)
+        .param('resample', 'If false, an existing level of the image is used '
+               'for the histogram.  If true, the internal values are '
+               'interpolated to match the specified size as needed.  0-3 for '
+               'a specific interpolation method (0-nearest, 1-lanczos, '
+               '2-bilinear, 3-bicubic)', required=False,
+               enum=['false', 'true', '0', '1', '2', '3'], default='false')
         .param('contentDisposition', 'Specify the Content-Disposition response '
                'header disposition-type value.', required=False,
                enum=['inline', 'attachment'])
@@ -704,6 +736,7 @@ class TilesItemResource(ItemResource):
             ('jpegSubsampling', int),
             ('tiffCompression', str),
             ('style', str),
+            ('resample', 'boolOrInt'),
             ('contentDisposition', str),
         ])
         _handleETag('getTilesRegion', item, params)
@@ -773,8 +806,10 @@ class TilesItemResource(ItemResource):
                default=2048, required=False, dataType='int')
         .param('resample', 'If false, an existing level of the image is used '
                'for the histogram.  If true, the internal values are '
-               'interpolated to match the specified size as needed.',
-               required=False, dataType='boolean', default=False)
+               'interpolated to match the specified size as needed.  0-3 for '
+               'a specific interpolation method (0-nearest, 1-lanczos, '
+               '2-bilinear, 3-bicubic)', required=False,
+               enum=['false', 'true', '0', '1', '2', '3'], default='false')
         .param('frame', 'For multiframe images, the 0-based frame number.  '
                'This is ignored on non-multiframe images.', required=False,
                dataType='int')
@@ -817,7 +852,7 @@ class TilesItemResource(ItemResource):
             ('jpegSubsampling', int),
             ('tiffCompression', str),
             ('style', str),
-            ('resample', bool),
+            ('resample', 'boolOrInt'),
             ('bins', int),
             ('rangeMin', int),
             ('rangeMax', int),
@@ -901,3 +936,28 @@ class TilesItemResource(ItemResource):
         setResponseHeader('Content-Type', imageMime)
         setRawResponse()
         return imageData
+
+    @autoDescribeRoute(
+        Description('Get metadata for an image associated with a large image.')
+        .modelParam('itemId', model=Item, level=AccessType.READ)
+        .param('image', 'The key of the associated image.', paramType='path')
+        .errorResponse('ID was invalid.')
+        .errorResponse('Read access was denied for the item.', 403)
+    )
+    @access.public
+    def getAssociatedImageMetadata(self, item, image, params):
+        _handleETag('getAssociatedImageMetadata', item, image)
+        tilesource = self.imageItemModel._loadTileSource(item, **params)
+        pilImage = tilesource._getAssociatedImage(image)
+        if pilImage is None:
+            return {}
+        result = {
+            'sizeX': pilImage.width,
+            'sizeY': pilImage.height,
+            'mode': pilImage.mode,
+        }
+        if pilImage.format:
+            result['format'] = pilImage.format
+        if pilImage.info:
+            result['info'] = pilImage.info
+        return result
