@@ -1,12 +1,9 @@
-# -*- coding: utf-8 -*-
-
 import math
-import mock
+from unittest import mock
 import os
 import pytest
 import requests
 import shutil
-import six
 import struct
 import time
 
@@ -27,7 +24,6 @@ from girder_large_image import getGirderTileSource
 from girder_large_image import loadmodelcache
 
 from . import girder_utilities as utilities
-from .girder_utilities import girderWorker, unbindLargeImage  # noqa
 
 
 def _testTilesZXY(server, admin, itemId, metadata, tileParams=None,
@@ -138,7 +134,7 @@ def _createTestTiles(server, admin, params=None, info=None, error=None):
     return infoDict
 
 
-def _postTileViaHttp(server, admin, itemId, fileId, jobAction=None):
+def _postTileViaHttp(server, admin, itemId, fileId, jobAction=None, data=None):
     """
     When we know we need to process a job, we have to use an actual http
     request rather than the normal simulated request to cherrypy.  This is
@@ -148,6 +144,9 @@ def _postTileViaHttp(server, admin, itemId, fileId, jobAction=None):
     :param itemId: the id of the item with the file to process.
     :param fileId: the id of the file that should be processed.
     :param jobAction: if 'delete', delete the job immediately.
+    :param data: if not None, pass this as the data to the POST request.  If
+        specified, fileId is ignored (pass as part of the data dictionary if
+        it is required).
     :returns: metadata from the tile if the conversion was successful,
               False if it converted but didn't result in useable tiles, and
               None if it failed.
@@ -158,20 +157,33 @@ def _postTileViaHttp(server, admin, itemId, fileId, jobAction=None):
     }
     req = requests.post('http://127.0.0.1:%d/api/v1/item/%s/tiles' % (
         server.boundPort, itemId), headers=headers,
-        data={'fileId': fileId})
+        data={'fileId': fileId} if data is None else data)
     assert req.status_code == 200
-    # If we ask to create the item again right away, we should be told that
-    # either there is already a job running or the item has already been
-    # added
-    req = requests.post('http://127.0.0.1:%d/api/v1/item/%s/tiles' % (
-        server.boundPort, itemId), headers=headers,
-        data={'fileId': fileId})
-    assert req.status_code == 400
-    assert ('Item already has' in req.json()['message'] or
-            'Item is scheduled' in req.json()['message'])
 
     if jobAction == 'delete':
         Job().remove(Job().find({}, sort=[('_id', SortDir.DESCENDING)])[0])
+        # Wait for the job to be complete
+        starttime = time.time()
+        while time.time() - starttime < 30:
+            req = requests.get(
+                'http://127.0.0.1:%d/api/v1/worker/status' % server.boundPort, headers=headers)
+            resp = req.json()
+            if resp.get('active') and not len(next(iter(resp['active'].items()))[1]):
+                resp = server.request(path='/item/%s/tiles' % itemId, user=admin)
+                if (utilities.respStatus(resp) == 400 and
+                        'No large image file' in resp.json['message']):
+                    break
+            time.sleep(0.1)
+    else:
+        # If we ask to create the item again right away, we should be told that
+        # either there is already a job running or the item has already been
+        # added
+        req = requests.post('http://127.0.0.1:%d/api/v1/item/%s/tiles' % (
+            server.boundPort, itemId), headers=headers,
+            data={'fileId': fileId} if data is None else data)
+        assert req.status_code == 400
+        assert ('Item already has' in req.json()['message'] or
+                'Item is scheduled' in req.json()['message'])
 
     starttime = time.time()
     resp = None
@@ -389,15 +401,13 @@ def testTilesFromTest(server, admin, fsAssetstore):
         _createTestTiles(server, admin, {key: badParams[key]}, error=err)
 
 
-@pytest.mark.usefixtures('unbindLargeImage')  # noqa
-@pytest.mark.usefixtures('girderWorker')  # noqa
+@pytest.mark.usefixtures('unbindLargeImage')
 @pytest.mark.plugin('large_image')
-def testTilesFromPNG(boundServer, admin, fsAssetstore, girderWorker):  # noqa
-    server = boundServer
+def testTilesFromPNG(boundServer, admin, fsAssetstore, girderWorker):
     file = utilities.uploadTestFile('yb10kx5k.png', admin, fsAssetstore)
     itemId = str(file['itemId'])
     fileId = str(file['_id'])
-    tileMetadata = _postTileViaHttp(server, admin, itemId, fileId)
+    tileMetadata = _postTileViaHttp(boundServer, admin, itemId, fileId)
     assert tileMetadata['tileWidth'] == 256
     assert tileMetadata['tileHeight'] == 256
     assert tileMetadata['sizeX'] == 10000
@@ -406,67 +416,68 @@ def testTilesFromPNG(boundServer, admin, fsAssetstore, girderWorker):  # noqa
     assert tileMetadata['magnification'] is None
     assert tileMetadata['mm_x'] is None
     assert tileMetadata['mm_y'] is None
-    _testTilesZXY(server, admin, itemId, tileMetadata)
+    _testTilesZXY(boundServer, admin, itemId, tileMetadata)
     # Ask to make this a tile-based item with an missing file ID (there are
     # now two files, so this will now fail).
-    resp = server.request(path='/item/%s/tiles' % itemId, method='POST',
-                          user=admin)
+    resp = boundServer.request(path='/item/%s/tiles' % itemId, method='POST', user=admin)
     assert utilities.respStatus(resp) == 400
     assert 'Missing "fileId"' in resp.json['message']
     # We should be able to delete the tiles
-    resp = server.request(path='/item/%s/tiles' % itemId, method='DELETE',
-                          user=admin)
+    resp = boundServer.request(path='/item/%s/tiles' % itemId, method='DELETE', user=admin)
     assert utilities.respStatus(resp) == 200
     assert resp.json['deleted'] is True
     # We should no longer have tile informaton
-    resp = server.request(path='/item/%s/tiles' % itemId, user=admin)
+    resp = boundServer.request(path='/item/%s/tiles' % itemId, user=admin)
     assert utilities.respStatus(resp) == 400
     assert 'No large image file' in resp.json['message']
     # This should work with a PNG with transparency, too.
     file = utilities.uploadTestFile('yb10kx5ktrans.png', admin, fsAssetstore)
     itemId = str(file['itemId'])
     fileId = str(file['_id'])
-    tileMetadata = _postTileViaHttp(server, admin, itemId, fileId)
+    tileMetadata = _postTileViaHttp(boundServer, admin, itemId, fileId)
     assert tileMetadata['tileWidth'] == 256
     assert tileMetadata['tileHeight'] == 256
     assert tileMetadata['sizeX'] == 10000
     assert tileMetadata['sizeY'] == 5000
     assert tileMetadata['levels'] == 7
-    _testTilesZXY(server, admin, itemId, tileMetadata)
+    _testTilesZXY(boundServer, admin, itemId, tileMetadata)
     # We should be able to delete the tiles
-    resp = server.request(path='/item/%s/tiles' % itemId, method='DELETE',
-                          user=admin)
+    resp = boundServer.request(path='/item/%s/tiles' % itemId, method='DELETE', user=admin)
     assert utilities.respStatus(resp) == 200
     assert resp.json['deleted'] is True
     # We should no longer have tile information
-    resp = server.request(path='/item/%s/tiles' % itemId, user=admin)
+    resp = boundServer.request(path='/item/%s/tiles' % itemId, user=admin)
     assert utilities.respStatus(resp) == 400
     assert 'No large image file' in resp.json['message']
+
+
+@pytest.mark.usefixtures('unbindLargeImage')
+@pytest.mark.plugin('large_image')
+def testTilesDeleteJob(boundServer, admin, fsAssetstore, girderWorker):
     # Make sure we don't auto-create a largeImage
-    file = utilities.uploadTestFile(
-        'yb10kx5k.png', admin, fsAssetstore, name='yb10kx5k.tiff')
+    file = utilities.uploadTestFile('yb10kx5k.png', admin, fsAssetstore, name='yb10kx5k.tiff')
     itemId = str(file['itemId'])
-    resp = server.request(path='/item/%s/tiles' % itemId, user=admin)
+    resp = boundServer.request(path='/item/%s/tiles' % itemId, user=admin)
     assert utilities.respStatus(resp) == 400
     assert 'No large image file' in resp.json['message']
 
     # Try to create an image, but delete the job and check that it fails.
     fileId = str(file['_id'])
-    result = _postTileViaHttp(server, admin, itemId, fileId, jobAction='delete')
+    result = _postTileViaHttp(boundServer, admin, itemId, fileId, jobAction='delete',
+                              data={'countdown': 10, 'fileId': fileId})
     assert result is None
     # If we end the test here, girder_worker may upload a file that gets
     # discarded, but do so in a manner that interfers with cleaning up the test
-    # temp directory.  By running another task, this is less likely to occur.
+    # temp directory.  By running other tasks, this is less likely to occur.
 
     # Creating it again should work
-    tileMetadata = _postTileViaHttp(server, admin, itemId, fileId)
+    tileMetadata = _postTileViaHttp(boundServer, admin, itemId, fileId)
     assert tileMetadata['levels'] == 7
 
 
-@pytest.mark.usefixtures('unbindLargeImage')  # noqa
-@pytest.mark.usefixtures('girderWorker')  # noqa
+@pytest.mark.usefixtures('unbindLargeImage')
 @pytest.mark.plugin('large_image')
-def testTilesFromGreyscale(boundServer, admin, fsAssetstore, girderWorker):  # noqa
+def testTilesFromGreyscale(boundServer, admin, fsAssetstore, girderWorker):
     file = utilities.uploadTestFile('grey10kx5k.tif', admin, fsAssetstore)
     itemId = str(file['itemId'])
     fileId = str(file['_id'])
@@ -482,22 +493,21 @@ def testTilesFromGreyscale(boundServer, admin, fsAssetstore, girderWorker):  # n
     _testTilesZXY(boundServer, admin, itemId, tileMetadata)
 
 
-@pytest.mark.usefixtures('unbindLargeImage')  # noqa
-@pytest.mark.usefixtures('girderWorker')  # noqa
+@pytest.mark.usefixtures('unbindLargeImage')
 @pytest.mark.plugin('large_image')
-def testTilesFromUnicodeName(boundServer, admin, fsAssetstore, girderWorker):  # noqa
+def testTilesFromUnicodeName(boundServer, admin, fsAssetstore, girderWorker):
     # Unicode file names shouldn't cause problems when generating tiles.
     file = utilities.uploadTestFile('yb10kx5k.png', admin, fsAssetstore)
     # Our normal testing method doesn't pass through the unicode name
     # properly, so just change it after upload.
     file = File().load(file['_id'], force=True)
-    file['name'] = u'\u0441\u043b\u0430\u0439\u0434'
+    file['name'] = '\u0441\u043b\u0430\u0439\u0434'
     file = File().save(file)
     fileId = str(file['_id'])
 
     itemId = str(file['itemId'])
     item = Item().load(itemId, force=True)
-    item['name'] = u'item \u0441\u043b\u0430\u0439\u0434'
+    item['name'] = 'item \u0441\u043b\u0430\u0439\u0434'
     item = Item().save(item)
 
     tileMetadata = _postTileViaHttp(boundServer, admin, itemId, fileId)
@@ -517,7 +527,7 @@ def testTilesFromUnicodeName(boundServer, admin, fsAssetstore, girderWorker):  #
 def testTilesWithUnicodeName(server, admin, fsAssetstore):
     # Unicode file names shouldn't cause problems when accessing ptifs.
     # This requires an appropriate version of the python libtiff module.
-    name = u'\u0441\u043b\u0430\u0439\u0434.ptif'
+    name = '\u0441\u043b\u0430\u0439\u0434.ptif'
     origpath = utilities.externaldata('data/sample_image.ptif.sha512')
     altpath = os.path.join(os.path.dirname(origpath), name)
     if os.path.exists(altpath):
@@ -534,10 +544,9 @@ def testTilesWithUnicodeName(server, admin, fsAssetstore):
     assert tileMetadata['sizeY'] == 12288
 
 
-@pytest.mark.usefixtures('unbindLargeImage')  # noqa
-@pytest.mark.usefixtures('girderWorker')  # noqa
+@pytest.mark.usefixtures('unbindLargeImage')
 @pytest.mark.plugin('large_image')
-def testTilesFromBadFiles(boundServer, admin, fsAssetstore, girderWorker):  # noqa
+def testTilesFromBadFiles(boundServer, admin, fsAssetstore, girderWorker):
     # As of vips 8.2.4, alpha and unusual channels are removed upon
     # conversion to a JPEG-compressed tif file.  Originally, we performed a
     # test to show that these files didn't work.  They now do (though if
@@ -851,7 +860,7 @@ def testTilesLoadModelCache(server, admin, fsAssetstore):
     tileMetadata = resp.json
     tileMetadata['sparse'] = 5
     _testTilesZXY(server, admin, itemId, tileMetadata, token=token)
-    assert six.next(six.itervalues(loadmodelcache.LoadModelCache))['hits'] > 70
+    assert next(iter(loadmodelcache.LoadModelCache.values()))['hits'] > 70
 
 
 @pytest.mark.usefixtures('unbindLargeImage')
@@ -986,9 +995,9 @@ def testTilesDZIEndpoints(server, admin, fsAssetstore):
     assert height == 260
 
 
-@pytest.mark.usefixtures('girderWorker')  # noqa
+@pytest.mark.usefixtures('unbindLargeImage')
 @pytest.mark.plugin('large_image')
-def testTilesAfterCopyItem(boundServer, admin, fsAssetstore, girderWorker):  # noqa
+def testTilesAfterCopyItem(boundServer, admin, fsAssetstore, girderWorker):
     file = utilities.uploadTestFile('yb10kx5k.png', admin, fsAssetstore)
     itemId = str(file['itemId'])
     fileId = str(file['_id'])
@@ -1036,7 +1045,7 @@ def testTilesAssociatedImages(server, admin, fsAssetstore):
 
     resp = server.request(path='/item/%s/tiles/images' % itemId, user=admin)
     assert utilities.respStatus(resp) == 200
-    assert resp.json, ['label' == 'macro']
+    assert resp.json == ['label', 'macro']
     resp = server.request(path='/item/%s/tiles/images/label' % itemId,
                           user=admin, isJson=False)
     assert utilities.respStatus(resp) == 200
@@ -1050,6 +1059,13 @@ def testTilesAssociatedImages(server, admin, fsAssetstore):
     assert image[:len(utilities.PNGHeader)] == utilities.PNGHeader
     (width, height) = struct.unpack('!LL', image[16:24])
     assert max(width, height) == 256
+    resp = server.request(path='/item/%s/tiles/images/label/metadata' % itemId,
+                          user=admin)
+    assert utilities.respStatus(resp) == 200
+    assert resp.json['sizeX'] == 819
+    assert resp.json['sizeY'] == 800
+    assert resp.json['format'] == 'JPEG'
+    assert resp.json['mode'] == 'RGB'
 
     # Test missing associated image
     resp = server.request(path='/item/%s/tiles/images/nosuchimage' % itemId,
@@ -1057,6 +1073,9 @@ def testTilesAssociatedImages(server, admin, fsAssetstore):
     assert utilities.respStatus(resp) == 200
     image = utilities.getBody(resp, text=False)
     assert image == b''
+    resp = server.request(path='/item/%s/tiles/images/nosuchimage/metadata' % itemId, user=admin)
+    assert utilities.respStatus(resp) == 200
+    assert resp.json == {}
 
     # Test with an image that doesn't have associated images
     file = utilities.uploadExternalFile(
@@ -1131,24 +1150,13 @@ def testTilesInternalMetadata(server, admin, fsAssetstore):
     assert resp.json['tilesource'] == 'tiff'
 
 
-@pytest.mark.usefixtures('unbindLargeImage')  # noqa
-@pytest.mark.usefixtures('girderWorker')  # noqa
+@pytest.mark.usefixtures('unbindLargeImage')
 @pytest.mark.plugin('large_image')
-def testTilesFromMultipleDotName(boundServer, admin, fsAssetstore, girderWorker):  # noqa
-    # Unicode file names shouldn't cause problems when generating tiles.
-    file = utilities.uploadTestFile('yb10kx5k.png', admin, fsAssetstore)
-    # Our normal testing method doesn't pass through the unicode name
-    # properly, so just change it after upload.
-    file = File().load(file['_id'], force=True)
-    file['name'] = 'A name with...dots.png'
-    file = File().save(file)
-    fileId = str(file['_id'])
-
+def testTilesFromMultipleDotName(boundServer, admin, fsAssetstore, girderWorker):
+    file = utilities.uploadTestFile(
+        'yb10kx5k.png', admin, fsAssetstore, name='A name with...dots.png')
     itemId = str(file['itemId'])
-    item = Item().load(itemId, force=True)
-    item['name'] = 'A name with...dots.png'
-    item = Item().save(item)
-
+    fileId = str(file['_id'])
     tileMetadata = _postTileViaHttp(boundServer, admin, itemId, fileId)
     assert tileMetadata['tileWidth'] == 256
     assert tileMetadata['tileHeight'] == 256
@@ -1159,3 +1167,34 @@ def testTilesFromMultipleDotName(boundServer, admin, fsAssetstore, girderWorker)
     assert tileMetadata['mm_x'] is None
     assert tileMetadata['mm_y'] is None
     _testTilesZXY(boundServer, admin, itemId, tileMetadata)
+
+
+@pytest.mark.usefixtures('unbindLargeImage')
+@pytest.mark.plugin('large_image')
+def testTilesForcedConversion(boundServer, admin, fsAssetstore, girderWorker):
+    file = utilities.uploadExternalFile(
+        'data/landcover_sample_1000.tif.sha512', admin, fsAssetstore)
+    itemId = str(file['itemId'])
+    fileId = str(file['_id'])
+    # We should already have tile information.  Ask to delete it so we can
+    # force convert it
+    boundServer.request(path='/item/%s/tiles' % itemId, method='DELETE', user=admin)
+    # Ask to do a forced conversion
+    tileMetadata = _postTileViaHttp(boundServer, admin, itemId, None, data={'force': True})
+    assert tileMetadata['levels'] == 3
+    item = Item().load(itemId, force=True)
+    assert item['largeImage']['fileId'] != fileId
+
+
+@pytest.mark.usefixtures('unbindLargeImage')
+@pytest.mark.plugin('large_image')
+def testTilesFromWithOptions(boundServer, admin, fsAssetstore, girderWorker):
+    file = utilities.uploadTestFile('yb10kx5k.png', admin, fsAssetstore)
+    itemId = str(file['itemId'])
+    fileId = str(file['_id'])
+    tileMetadata = _postTileViaHttp(boundServer, admin, itemId, fileId, data={'tileSize': 1024})
+    assert tileMetadata['tileWidth'] == 1024
+    assert tileMetadata['tileHeight'] == 1024
+    assert tileMetadata['sizeX'] == 10000
+    assert tileMetadata['sizeY'] == 5000
+    assert tileMetadata['levels'] == 5
